#include "HPMA.h"
#include <cstdint>
#include <cstdio>

HPMA::HPMA(PinName TX, PinName RX, int BaudRate) : device(TX, RX, BaudRate) {}

//Error code:   -1  -> Unknown error
//              -10 -> Read error
//              -11 -> Checksum error
HPMAPDU* HPMA::read() {
    printf("Start reading command\n");
    HPMAPDU* response = new HPMAPDU();
    unsigned char Request[] = { 0x68, 0x01, READ_COMMAND ,0x96};
    Request[3] = MOD(Request, 3, 256);
    device.write(Request, 4);
    thread_sleep_for(50);
    unsigned char buffer[MAX_BUFFER_SIZE] = {0};
    device.read(buffer, MAX_BUFFER_SIZE);
    print_array(buffer, sizeof(buffer));
    if(buffer[0] == NEG_ACK && buffer[1] == NEG_ACK)  {
        response->PM25 = -10;
        response->PM10 = -10;
        
    }
    else if(buffer[0] == START_ACK_BYTE && buffer[1] == READ_LENGTH && buffer[2] == READ_COMMAND) {
        if(buffer[7] == MOD(buffer, 7, 256)) {
            response->PM25 = buffer[3] * 256 + buffer[4];
            response->PM10 = buffer[5] * 256 + buffer[6];
        }
        else {
            response->PM25 = -11;
            response->PM10 = -11;
        }
        
    }
    else {
        response->PM25 = -1;
        response->PM10 = -1;
    }
    return response;
}

bool HPMA::start_measurement() {
    printf("Start command\n");
    unsigned char StartRequest[] = { START_BYTE, 0x01, START_COMMAND, 0x96 };
    StartRequest[3] = MOD(StartRequest, 3, 256);
    device.write(StartRequest, sizeof(StartRequest));
    thread_sleep_for(50);
    unsigned char buffer[2];
    device.read(buffer, sizeof(buffer));
    if ((buffer[0] == POS_ACK) && (buffer[1] == POS_ACK)) {
        return true;
    }
    else if ((buffer[0] == NEG_ACK) && (buffer[1] == NEG_ACK)) {
        return false;
    }
    else return false;
}
bool HPMA::stop_measurement() {
    printf("Stop command\n");
    unsigned char StopRequest[] = { START_BYTE, 0x01, STOP_COMMAND, 0x96 };
    StopRequest[3] = MOD(StopRequest, 3, 256);
    device.write(StopRequest, sizeof(StopRequest));
    thread_sleep_for(50);
    unsigned char buffer[2];
    device.read(buffer, sizeof(buffer));
    if ((buffer[0] == POS_ACK) && (buffer[1] == POS_ACK)) {
        return true;
    }
    else if ((buffer[0] == NEG_ACK) && (buffer[1] == NEG_ACK)) {
        return false;
    }
    else return false;
}
bool HPMA::stop_autosend() {
    printf("Stop auto send command\n");
    unsigned char StopAutoSendRequest[] = { START_BYTE, 0x01, STOP_AUTO_SEND_COMMAND, 0x96 };
    StopAutoSendRequest[3] = MOD(StopAutoSendRequest, 3, 256);
    device.write(StopAutoSendRequest, sizeof(StopAutoSendRequest));
    thread_sleep_for(50);
    unsigned char buffer[2];
    device.read(buffer, sizeof(buffer));
    if ((buffer[0] == POS_ACK) && (buffer[1] == POS_ACK)) {
        return true;
    }
    else if ((buffer[0] == NEG_ACK) && (buffer[1] == NEG_ACK)) {
        return false;
    }
    else return false;
}
bool HPMA::enable_autosend() {
    printf("Enable auto send command\n");
    unsigned char EnableAutoSendRequest[] = { START_BYTE, 0x01, ENABLE_AUTO_SEND_COMMAND, 0x96 };
    EnableAutoSendRequest[3] = MOD(EnableAutoSendRequest, 3, 256);
    device.write(EnableAutoSendRequest, sizeof(EnableAutoSendRequest));
    thread_sleep_for(50);
    unsigned char buffer[2];
    device.read(buffer, sizeof(buffer));
    if ((buffer[0] == POS_ACK) && (buffer[1] == POS_ACK)) {
        return true;
    }
    else if ((buffer[0] == NEG_ACK) && (buffer[1] == NEG_ACK)) {
        return false;
    }
    else return false;
}

bool HPMA::set_customer_adjust_coef(unsigned char coef) {
    printf("Enable auto send command\n");

    if(coef < 30 || coef > 200) return false;

    unsigned char SetCusAdjCoefRequest[] = { START_BYTE, 0x02, SET_CUSTOMER__ADJUST_COEF_COMMAND, coef, 0x96 };
    SetCusAdjCoefRequest[3] = MOD(SetCusAdjCoefRequest, 3, 256);
    device.write(SetCusAdjCoefRequest, sizeof(SetCusAdjCoefRequest));
    thread_sleep_for(50);

    unsigned char buffer[2];
    device.read(buffer, sizeof(buffer));
    if ((buffer[0] == POS_ACK) && (buffer[1] == POS_ACK)) {
        return true;
    }
    else if ((buffer[0] == NEG_ACK) && (buffer[1] == NEG_ACK)) {
        return false;
    }
    else return false;
}
//Error code:   -1  -> Onbekende fout
//              -10 -> Lees error
//              -11 -> Checksum error
int HPMA::read_customer_adjust_coef() {
    printf("Start reading command\n");
    unsigned char ReadCusAdjCoefRequest[] = { START_BYTE, 0x01, READ_CUSTOMER_ADJUST_COEF_COMMAND, 0x96 };
    ReadCusAdjCoefRequest[3] = MOD(ReadCusAdjCoefRequest, 3, 256);
    device.write(ReadCusAdjCoefRequest, 4);
    thread_sleep_for(50);
    unsigned char buffer[MAX_BUFFER_SIZE] = {0};
    device.read(buffer, MAX_BUFFER_SIZE);
    print_array(buffer, sizeof(buffer));
    if(buffer[0] == NEG_ACK && buffer[1] == NEG_ACK)  {
        return -10;
    }
    else if(buffer[0] == START_ACK_BYTE && buffer[1] == READ_COEF_LENGTH && buffer[2] == READ_CUSTOMER_ADJUST_COEF_COMMAND) {
        if(buffer[4] == MOD(buffer, 4, 256)) {
            return buffer[3];
        }
        else {
            return -11;
        }
    }
    else {
        return -1;
    }
}

unsigned char HPMA::MOD(unsigned char* list, int length, int mod) {
    int val = *list;
    for (int i = 1; i < length; i++)
        val += list[i];
    unsigned char temp = (unsigned char)((65536 - val) % mod);
    printf("CheckSum: %d\n", temp);
    return temp;
}