#include "mbed.h"
#include "par.h"

// List of all the commands the HPMA sensor can receive or send out
#define NEG_ACK 0X96
#define POS_ACK 0xA5
#define POS_ACK_HEAD 0x40
#define START_BYTE 0x68
#define START_ACK_BYTE 0x40
#define START_COMMAND 0x01
#define STOP_COMMAND 0x02
#define READ_COMMAND 0x04
#define READ_LENGTH 0x05
#define READ_COEF_LENGTH 0x02
#define SET_CUSTOMER__ADJUST_COEF_COMMAND 0x08
#define READ_CUSTOMER_ADJUST_COEF_COMMAND 0x10
#define STOP_AUTO_SEND_COMMAND 0x20
#define ENABLE_AUTO_SEND_COMMAND 0x40
#define MAX_BUFFER_SIZE 8

// Struct for the read function
// It stores the output data
struct HPMAPDU {
    int PM25;
    int PM10;
};

// The HPMA class it handles everything the sensor needs to do
class HPMA {
    private:
        // A private function to calculate the checksum for sending and receiving
        unsigned char MOD(unsigned char* list, int length, int mod);
    public:
        // The Serial interface in this instance UART
        BufferedSerial device;

        // The constructor that takes the Serial interface TX and RX pins and optional baudrate
        // Baudrate should normally be 9600Hz for the HPMA sensor
        HPMA(PinName TX, PinName RX, int BaudRate = 9600);

        // This function stops the fan of the sensor. Therefore it doesn't blow in new air
        // You should not read after calling this function. Your measurements will be incorrect
        // This functions returns true if succeeded, false if an error occurred
        bool stop_measurement();

        // This function starts the fan. Therefore allowing air and particles to the sensor.
        // You should wait a bit before reading otherwise you risk measering wrong.
        // This functions returns true if succeeded, false if an error occurred
		bool start_measurement();

        // The sensor has autosend capabilities. This function stops it.
        // This functions returns true if succeeded, false if an error occurred
		bool stop_autosend();

        // The sensor has autosend capabilities. This function starts it.
        // This functions returns true if succeeded, false if an error occurred
		bool enable_autosend();
        
        // This function will read the current amount of particles and return a HPMAPDU struct.
        // In this struct you will find the PM2.5 and PM10 values.
        // If the values inside the struct are one of the following:
        // Error code:   -1  -> Unknown error
        //               -10 -> Read error
        //               -11 -> Checksum error
        // Then an error has occurred
        HPMAPDU* read();

        // You can set a customer adjustmend coëfficient with these 2 functions
        bool set_customer_adjust_coef(unsigned char coef);
        int read_customer_adjust_coef();
};

